import createPub from '../';

type QueryFuncType = (...args: Array<any>) => Promise;

type CreateQueryPubOptions = {
    queryFunc?: QueryFuncType,
    cacheTimeout?: number,
    hidePendingOnRefresh?: boolean
    avoidRepeatInProgress?: boolean,
}

enum PromiseStatus  {
    PENDING = "PENDING",
    FULFILLED = "FULFILLED",
    REJECTED = "REJECTED"
}

type PromiseSnapshot = {
    status: PromiseStatus,
    payload: any,
    error: Error
}

const defaultOptions:CreateQueryPubOptions = {
    cacheTimeout: 2 * 60 * 1000,
    hidePendingOnRefresh: true,
    avoidRepeatInProgress: true,
}

export default (options:CreateQueryPubOptions) => {
    const queryPub = createPub({queries: {}, mutations: {}})

    options = {...defaultOptions, ...options}

    const createQuery = (queryFunc: QueryFuncType):Function => {
        return (promiseName:string, isMutation:boolean) => {
            const qF = queryFunc || options.queryFunc

            const forceQuery = (...args) => {
                return qF(...args)
            }

            const query = (...args) => {
                if (isMutation && (!options.avoidRepeatInProgress || queryPub.mutations?.[promiseName]?.status !== 'PENDING')){
                    queryPub.mutations[promiseName] = {status: 'PENDING'}
                    return forceQuery(...args).then(payload => queryPub.mutations[promiseName] = {status: 'FULFILLED', payload},
                                                    error   => queryPub.mutations[promiseName] = {status: 'REJECTED', error})
                }
                if (!isMutation){
                    const stringifiedArgs = JSON.stringify(args)
                    queryPub[promiseName][stringifiedArgs]
                }
            }
            return {forceQuery, query}
        }
    }

    return {queryPub, createQuery}
}
